@@ -1,6 +1,6 @@
 /*
  *  Copyright 2001-2007 Internet2
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -124,7 +124,7 @@ public:
             cerr << "shib: " << msg;
     }
     void clearHeader(const char* rawname, const char* cginame) {
-        // no need, since request headers turn into actual environment variables
+        // No need, since we use environment variables.
     }
     void setHeader(const char* name, const char* value) {
         if (value)
@@ -132,12 +132,28 @@ public:
         else
             m_request_headers.erase(name);
     }
-    virtual string getHeader(const char* name) const {
+    string getHeader(const char* name) const {
+        // Look in the local map first.
         map<string,string>::const_iterator i = m_request_headers.find(name);
         if (i != m_request_headers.end())
             return i->second;
-        else
-            return "";
+        // Nothing set locally and this isn't a "secure" call, so check the request.
+        string hdr("HTTP_");
+        for (; *name; ++name) {
+            if (*name=='-')
+                hdr += '_';
+            else
+                hdr += toupper(*name);
+        }
+        char* s = FCGX_GetParam(hdr.c_str(), m_req->envp);
+        return s ? s : "";
+    }
+    string getSecureHeader(const char* name) const {
+        // Look in the local map only.
+        map<string,string>::const_iterator i = m_request_headers.find(name);
+        if (i != m_request_headers.end())
+            return i->second;
+        return "";
     }
     void setRemoteUser(const char* user) {
         if (user)
@@ -169,7 +185,7 @@ public:
     const char* getRequestBody() const {
         throw runtime_error("getRequestBody not implemented by FastCGI authorizer.");
     }
- 
+
     long sendResponse(istream& in, long status) {
         string hdr = string("Connection: close\r\n");
         for (multimap<string,string>::const_iterator i=m_response_headers.begin(); i!=m_response_headers.end(); ++i)
@@ -206,7 +222,7 @@ public:
         return SHIB_RETURN_DONE;
     }
 
-    long returnDecline() { 
+    long returnDecline() {
         return SHIB_RETURN_KO;
     }
 
@@ -250,20 +266,9 @@ int main(void)
         exit(1);
     }
 
-    const char* config=getenv("SHIBSP_CONFIG");
-    if (!config)
-        config=SHIBSP_CONFIG;
-
     try {
-        DOMDocument* dummydoc=XMLToolingConfig::getConfig().getParser().newDocument();
-        XercesJanitor<DOMDocument> docjanitor(dummydoc);
-        DOMElement* dummy = dummydoc->createElementNS(NULL,path);
-        auto_ptr_XMLCh src(config);
-        dummy->setAttributeNS(NULL,path,src.get());
-        dummy->setAttributeNS(NULL,validate,xmlconstants::XML_ONE);
-
-        g_Config->setServiceProvider(g_Config->ServiceProviderManager.newPlugin(XML_SERVICE_PROVIDER,dummy));
-        g_Config->getServiceProvider()->init();
+        if (!g_Config->instantiate(NULL, true))
+            throw runtime_error("unknown error");
     }
     catch (exception& ex) {
         g_Config->term();
@@ -293,7 +298,7 @@ int main(void)
 
     FCGX_Init();
     FCGX_InitRequest(&request, 0, 0);
-    
+
     cout << "Shibboleth initialization complete. Starting request loop." << endl;
     while (FCGX_Accept_r(&request) == 0)
     {
@@ -309,48 +314,44 @@ int main(void)
         try {
             xmltooling::NDC ndc("FastCGI shibauthorizer");
             ShibTargetFCGIAuth sta(&request, g_ServerScheme.c_str(), g_ServerName.c_str(), g_ServerPort);
-          
+
             pair<bool,long> res = sta.getServiceProvider().doAuthentication(sta);
             if (res.first) {
-#ifdef _DEBUG
-                cerr << "shib: doAuthentication handled the request" << endl;
-#endif
+                sta.log(SPRequest::SPDebug, "shib: doAuthentication handled the request");
                 switch(res.second) {
                     case SHIB_RETURN_OK:
                         print_ok(sta.m_request_headers);
                         continue;
-              
+
                     case SHIB_RETURN_KO:
                         print_ok(sta.m_request_headers);
                         continue;
 
                     case SHIB_RETURN_DONE:
                         continue;
-              
+
                     default:
                         cerr << "shib: doAuthentication returned an unexpected result: " << res.second << endl;
                         print_error("<html><body>FastCGI Shibboleth authorizer returned an unexpected result.</body></html>");
                         continue;
                 }
             }
-          
+
             res = sta.getServiceProvider().doExport(sta);
             if (res.first) {
-#ifdef _DEBUG
-                cerr << "shib: doExport handled request" << endl;
-#endif
+                sta.log(SPRequest::SPDebug, "shib: doExport handled request");
                 switch(res.second) {
                     case SHIB_RETURN_OK:
                         print_ok(sta.m_request_headers);
                         continue;
-              
+
                     case SHIB_RETURN_KO:
                         print_ok(sta.m_request_headers);
                         continue;
 
                     case SHIB_RETURN_DONE:
                         continue;
-              
+
                     default:
                         cerr << "shib: doExport returned an unexpected result: " << res.second << endl;
                         print_error("<html><body>FastCGI Shibboleth authorizer returned an unexpected result.</body></html>");
@@ -360,21 +361,19 @@ int main(void)
 
             res = sta.getServiceProvider().doAuthorization(sta);
             if (res.first) {
-#ifdef _DEBUG
-                cerr << "shib: doAuthorization handled request" << endl;
-#endif
+                sta.log(SPRequest::SPDebug, "shib: doAuthorization handled request");
                 switch(res.second) {
                     case SHIB_RETURN_OK:
                         print_ok(sta.m_request_headers);
                         continue;
-              
+
                     case SHIB_RETURN_KO:
                         print_ok(sta.m_request_headers);
                         continue;
 
                     case SHIB_RETURN_DONE:
                         continue;
-              
+
                     default:
                         cerr << "shib: doAuthorization returned an unexpected result: " << res.second << endl;
                         print_error("<html><body>FastCGI Shibboleth authorizer returned an unexpected result.</body></html>");
@@ -383,7 +382,7 @@ int main(void)
             }
 
             print_ok(sta.m_request_headers);
-          
+
         }
         catch (exception& e) {
             cerr << "shib: FastCGI authorizer caught an exception: " << e.what() << endl;
@@ -402,6 +401,6 @@ int main(void)
 
     if (g_Config)
         g_Config->term();
- 
+
     return 0;
 }
