@@ -1,6 +1,6 @@
 /*
  *  Copyright 2001-2007 Internet2
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -209,7 +209,7 @@ public:
     virtual void clearHeader(const char* rawname, const char* cginame) {
         throw runtime_error("clearHeader not implemented by FastCGI responder.");
     }
-  
+
     virtual void setHeader(const char* name, const char* value) {
         throw runtime_error("setHeader not implemented by FastCGI responder.");
     }
@@ -283,20 +283,9 @@ int main(void)
         exit(1);
     }
 
-    const char* config=getenv("SHIBSP_CONFIG");
-    if (!config)
-        config=SHIBSP_CONFIG;
-
     try {
-        DOMDocument* dummydoc=XMLToolingConfig::getConfig().getParser().newDocument();
-        XercesJanitor<DOMDocument> docjanitor(dummydoc);
-        DOMElement* dummy = dummydoc->createElementNS(NULL,path);
-        auto_ptr_XMLCh src(config);
-        dummy->setAttributeNS(NULL,path,src.get());
-        dummy->setAttributeNS(NULL,validate,xmlconstants::XML_ONE);
-
-        g_Config->setServiceProvider(g_Config->ServiceProviderManager.newPlugin(XML_SERVICE_PROVIDER,dummy));
-        g_Config->getServiceProvider()->init();
+        if (!g_Config->instantiate(NULL, true))
+            throw runtime_error("unknown error");
     }
     catch (exception& ex) {
         g_Config->term();
@@ -327,7 +316,7 @@ int main(void)
 
     FCGX_Init();
     FCGX_InitRequest(&request, 0, 0);
-    
+
     cout << "Shibboleth initialization complete. Starting request loop." << endl;
     while (FCGX_Accept_r(&request) == 0) {
         // Note that the default bufsize (0) will cause the use of iostream
@@ -350,17 +339,15 @@ int main(void)
         try {
             xmltooling::NDC ndc("FastCGI shibresponder");
             ShibTargetFCGI stf(&request, content, g_ServerScheme.c_str(), g_ServerName.c_str(), g_ServerPort);
-          
+
             pair<bool,long> res = stf.getServiceProvider().doHandler(stf);
             if (res.first) {
-#ifdef _DEBUG
-                cerr << "shib: doHandler handled the request" << endl;
-#endif
+                stf.log(SPRequest::SPDebug, "shib: doHandler handled the request");
                 switch(res.second) {
                     case SHIB_RETURN_OK:
                         print_ok();
                         break;
-              
+
                     case SHIB_RETURN_KO:
                         cerr << "shib: doHandler failed to handle the request" << endl;
                         print_error("<html><body>FastCGI Shibboleth responder should only be used for Shibboleth protocol requests.</body></html>");
@@ -369,7 +356,7 @@ int main(void)
                     case SHIB_RETURN_DONE:
                         // response already handled
                         break;
-              
+
                     default:
                         cerr << "shib: doHandler returned an unexpected result: " << res.second << endl;
                         print_error("<html><body>FastCGI Shibboleth responder returned an unexpected result.</body></html>");
@@ -379,8 +366,8 @@ int main(void)
             else {
                 cerr << "shib: doHandler failed to handle request." << endl;
                 print_error("<html><body>FastCGI Shibboleth responder failed to process request.</body></html>");
-            }          
-          
+            }
+
         }
         catch (exception& e) {
             cerr << "shib: FastCGI responder caught an exception: " << e.what() << endl;
@@ -403,6 +390,6 @@ int main(void)
 
     if (g_Config)
         g_Config->term();
- 
+
     return 0;
 }
